@@ -1,6 +1,6 @@
 /* srfi-13.c --- SRFI-13 procedures for Guile
  *
- * Copyright (C) 2001, 2004, 2005, 2006, 2008, 2009, 2010, 2011 Free Software Foundation, Inc.
+ * Copyright (C) 2001, 2004, 2005, 2006, 2008, 2009, 2010, 2011, 2012 Free Software Foundation, Inc.
  *
  * This library is free software; you can redistribute it and/or
  * modify it under the terms of the GNU Lesser General Public License
@@ -375,7 +375,7 @@ SCM_SYMBOL (scm_sym_prefix, "prefix");
 SCM_DEFINE (scm_string_join, "string-join", 1, 2, 0,
             (SCM ls, SCM delimiter, SCM grammar),
 	    "Append the string in the string list @var{ls}, using the string\n"
-	    "@var{delim} as a delimiter between the elements of @var{ls}.\n"
+	    "@var{delimiter} as a delimiter between the elements of @var{ls}.\n"
 	    "@var{grammar} is a symbol which specifies how the delimiter is\n"
 	    "placed between the strings, and defaults to the symbol\n"
 	    "@code{infix}.\n"
@@ -719,7 +719,8 @@ SCM_DEFINE (scm_string_trim, "string-trim", 1, 3, 0,
   MY_VALIDATE_SUBSTRING_SPEC (1, s,
 			      3, start, cstart,
 			      4, end, cend);
-  if (SCM_UNBNDP (char_pred))
+  if (SCM_UNBNDP (char_pred)
+      || scm_is_eq (char_pred, scm_char_set_whitespace))
     {
       while (cstart < cend)
 	{
@@ -794,7 +795,8 @@ SCM_DEFINE (scm_string_trim_right, "string-trim-right", 1, 3, 0,
   MY_VALIDATE_SUBSTRING_SPEC (1, s,
 			      3, start, cstart,
 			      4, end, cend);
-  if (SCM_UNBNDP (char_pred))
+  if (SCM_UNBNDP (char_pred)
+      || scm_is_eq (char_pred, scm_char_set_whitespace))
     {
       while (cstart < cend)
 	{
@@ -869,7 +871,8 @@ SCM_DEFINE (scm_string_trim_both, "string-trim-both", 1, 3, 0,
   MY_VALIDATE_SUBSTRING_SPEC (1, s,
 			      3, start, cstart,
 			      4, end, cend);
-  if (SCM_UNBNDP (char_pred))
+  if (SCM_UNBNDP (char_pred)
+      || scm_is_eq (char_pred, scm_char_set_whitespace))
     {
       while (cstart < cend)
 	{
@@ -1356,7 +1359,7 @@ SCM_DEFINE (scm_string_ci_ge, "string-ci>=", 2, 4, 0,
 
 SCM_DEFINE (scm_substring_hash, "string-hash", 1, 3, 0,
 	    (SCM s, SCM bound, SCM start, SCM end),
-	    "Compute a hash value for @var{S}.  the optional argument "
+	    "Compute a hash value for @var{s}.  the optional argument "
 	    "@var{bound} is a non-negative exact "
             "integer specifying the range of the hash function. "
 	    "A positive value restricts the return value to the "
@@ -1373,7 +1376,7 @@ SCM_DEFINE (scm_substring_hash, "string-hash", 1, 3, 0,
 
 SCM_DEFINE (scm_substring_hash_ci, "string-hash-ci", 1, 3, 0,
 	    (SCM s, SCM bound, SCM start, SCM end),
-	    "Compute a hash value for @var{S}.  the optional argument "
+	    "Compute a hash value for @var{s}.  the optional argument "
 	    "@var{bound} is a non-negative exact "
             "integer specifying the range of the hash function. "
 	    "A positive value restricts the return value to the "
